"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const wd = require("wd");
const webdriverio = require("webdriverio");
const chai = require("chai");
const chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);
exports.should = chai.should();
chaiAsPromised.transferPromiseness = wd.transferPromiseness;
const element_helper_1 = require("./element-helper");
const search_options_1 = require("./search-options");
const ui_element_1 = require("./ui-element");
const locators_1 = require("./locators");
const mobile_devices_controller_1 = require("mobile-devices-controller");
const utils_1 = require("./utils");
const image_helper_1 = require("./image-helper");
const fs_1 = require("fs");
const device_manager_1 = require("../lib/device-manager");
const path_1 = require("path");
const log_types_1 = require("./log-types");
class AppiumDriver {
    constructor(_driver, _wd, _webio, _driverConfig, _args) {
        this._driver = _driver;
        this._wd = _wd;
        this._webio = _webio;
        this._driverConfig = _driverConfig;
        this._args = _args;
        this._defaultWaitTime = 5000;
        this._isAlive = false;
        this._elementHelper = new element_helper_1.ElementHelper(this._args);
        this._imageHelper = new image_helper_1.ImageHelper(this._args);
        this._isAlive = true;
        this._locators = new locators_1.Locator(this._args);
        this._webio.requestHandler.sessionID = this._driver.sessionID;
    }
    get imageHelper() {
        return this._imageHelper;
    }
    get defaultWaitTime() {
        return this._defaultWaitTime;
    }
    set defaultWaitTime(waitTime) {
        this._defaultWaitTime = waitTime;
    }
    get capabilities() {
        return this._args.appiumCaps;
    }
    get nsCapabilities() {
        return this._args;
    }
    get platformName() {
        return this._args.appiumCaps.platformName;
    }
    get platformVersion() {
        return this._args.appiumCaps.platformVersion;
    }
    get elementHelper() {
        return this._elementHelper;
    }
    get locators() {
        return this._locators;
    }
    get isAlive() {
        return this._isAlive;
    }
    get isAndroid() {
        return this._args.isAndroid;
    }
    get isIOS() {
        return this._args.isIOS;
    }
    get driver() {
        return this._driver;
    }
    /**
    * Get the storage where test results from image comparisson is logged It will be reports/app nam/device name
    */
    get reportsPath() {
        return this._logPath;
    }
    /**
    * Get the storage where images are captured by platform. It will be resources/app nam/platform name
    */
    get storageByPlatform() {
        return this._storageByPlatform;
    }
    /**
     * Get the storage where images are captured. It will be resources/app nam/device name
     */
    get storageByDeviceName() {
        return this._storageByDeviceName;
    }
    set storageByDeviceName(storageFullPath) {
        this._storageByDeviceName = storageFullPath;
    }
    get storage() {
        return utils_1.getStorage(this._args);
    }
    /**
     * Returns instance of wd.TouchAction object
     */
    get touchAction() {
        return new this._wd.TouchAction(this._driver);
    }
    webio() {
        return this._webio;
    }
    wd() {
        return this._wd;
    }
    click(args) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._webio.click(args);
        });
    }
    navBack() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._driver.back();
        });
    }
    // Still not supported in wd
    // public async getPerformanceDataTypes() {
    //     return await this._driver.getSupportedPerformanceDataTypes;
    // }
    // public async getPerformanceData(type, timeout: number = 5) {
    //     return await this._driver.getPerformanceData(this._args.appiumCaps.appPackage, type, timeout);
    // }
    static createAppiumDriver(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!args.isValidated) {
                yield args.validateArgs();
            }
            utils_1.log("Creating driver!", args.verbose);
            if (!args.attachToDebug && !args.sessionId) {
                if (!args.device) {
                    args.deviceManager = args.deviceManager || new device_manager_1.DeviceManager();
                    yield utils_1.prepareDevice(args, args.deviceManager);
                    yield utils_1.prepareApp(args);
                }
                yield AppiumDriver.applyAdditionalSettings(args);
            }
            const webio = webdriverio.remote({
                baseUrl: args.driverConfig.host,
                port: args.driverConfig.port,
                desiredCapabilities: args.appiumCaps
            });
            const driver = yield wd.promiseChainRemote(args.driverConfig);
            AppiumDriver.configureLogging(driver, args.verbose);
            let hasStarted = false;
            let retries = 10;
            while (retries > 0 && !hasStarted) {
                try {
                    let sessionInfo;
                    try {
                        if (args.sessionId || args.attachToDebug) {
                            const sessionInfos = JSON.parse(((yield utils_1.getSessions(args.port)) || "{}") + '');
                            sessionInfo = sessionInfos.value.filter(value => args.sessionId ? args.sessionId === value.id : true)[0];
                            if (!sessionInfo || !sessionInfo.id) {
                                utils_1.logError("No suitable session info found", sessionInfo);
                                process.exit(1);
                            }
                            args.sessionId = sessionInfo.id;
                            args.appiumCaps = sessionInfo.capabilities;
                            // remove app to prevent appium from installing app again
                            args.appiumCaps.app = "";
                            if (sessionInfo.capabilities.automationName) {
                                args.setAutomationNameFromString(sessionInfo.capabilities.automationName);
                            }
                            utils_1.prepareApp(args);
                            if (!args.device) {
                                if (args.isAndroid) {
                                    args.device = device_manager_1.DeviceManager.getDefaultDevice(args, sessionInfo.capabilities.avd, sessionInfo.capabilities.deviceUDID.replace("emulator-", ""), sessionInfo.capabilities.deviceUDID.includes("emulator") ? mobile_devices_controller_1.DeviceType.EMULATOR : mobile_devices_controller_1.DeviceType.SIMULATOR, sessionInfo.capabilities.desired.platformVersion || sessionInfo.capabilities.platformVersion);
                                }
                                else {
                                    args.device = device_manager_1.DeviceManager.getDefaultDevice(args);
                                }
                            }
                            yield driver.attach(args.sessionId);
                        }
                        else {
                            sessionInfo = yield driver.init(args.appiumCaps);
                        }
                    }
                    catch (error) {
                        args.verbose = true;
                        if (!args.ignoreDeviceController && error && error.message && error.message.includes("Failure [INSTALL_FAILED_INSUFFICIENT_STORAGE]")) {
                            yield device_manager_1.DeviceManager.kill(args.device);
                            yield mobile_devices_controller_1.DeviceController.startDevice(args.device);
                        }
                    }
                    if (args.verbose) {
                        utils_1.logInfo("Session info");
                        console.info(sessionInfo);
                    }
                    yield device_manager_1.DeviceManager.applyDeviceAdditionsSettings(driver, args, sessionInfo);
                    hasStarted = true;
                }
                catch (error) {
                    console.log(error);
                    console.log("Retry launching appium driver!");
                    hasStarted = false;
                    if (error && error.message && error.message.includes("WebDriverAgent")) {
                        const freePort = yield utils_1.findFreePort(100, args.wdaLocalPort);
                        console.log("args.appiumCaps['wdaLocalPort']", freePort);
                        args.appiumCaps["wdaLocalPort"] = freePort;
                    }
                }
                if (hasStarted) {
                    console.log("Appium driver has started successfully!");
                }
                else {
                    utils_1.logError("Appium driver is NOT started!");
                }
                retries--;
            }
            return new AppiumDriver(driver, wd, webio, args.driverConfig, args);
        });
    }
    /**
     *
     * @param xPath
     * @param waitForElement
     */
    findElementByXPath(xPath, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            const searchM = "waitForElementByXPath";
            return yield new ui_element_1.UIElement(yield this._driver.waitForElementByXPath(xPath, waitForElement), this._driver, this._wd, this._webio, this._args, searchM, xPath);
        });
    }
    /**
     *
     * @param xPath
     * @param waitForElement
     */
    findElementsByXPath(xPath, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.convertArrayToUIElements(yield this._driver.waitForElementsByXPath(xPath, waitForElement), "waitForElementsByXPath", xPath);
        });
    }
    /**
     * Search for element by given text. The search is case insensitive for android
     * @param text
     * @param match
     * @param waitForElement
     */
    findElementByText(text, match = search_options_1.SearchOptions.exact, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldMatch = match === search_options_1.SearchOptions.exact ? true : false;
            return yield this.findElementByXPath(this._elementHelper.getXPathByText(text, shouldMatch), waitForElement);
        });
    }
    /**
     * Search for element by given automationText. Searches only for exact string.
     * @param text
     * @param waitForElement
     */
    findElementByAutomationText(automationText, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isIOS) {
                return yield this.findElementByAccessibilityId(`${automationText}`, waitForElement);
            }
            return yield this.findElementByXPath(this._elementHelper.getXPathByText(automationText, true), waitForElement);
        });
    }
    /**
     * Search for element by given automationText. Searches only for exact string.
     * @param text
     * @param waitForElement
    */
    findElementByAutomationTextIfExists(automationText, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isIOS) {
                return yield this.findElementByAccessibilityIdIfExists(`${automationText}`, waitForElement);
            }
            return yield this.findElementByXPathIfExists(this._elementHelper.getXPathByText(automationText, true), waitForElement);
        });
    }
    /**
     * Search for element by given automationText and waits until the element is displayed.
     * @param text
     * @param waitInMilliseconds till element is displayed
     */
    waitForElement(automationText, waitInMilliseconds = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            let element;
            try {
                element = yield this.findElementByAutomationTextIfExists(automationText, 100);
            }
            catch (error) { }
            const startTime = Date.now();
            while ((!element || !(yield element.isDisplayed())) && Date.now() - startTime <= waitInMilliseconds) {
                try {
                    element = yield this.findElementByAutomationTextIfExists(automationText, 100);
                }
                catch (error) { }
            }
            if (!element || !(yield element.isDisplayed())) {
                const msg = `Element with automationText: '${automationText}' is not displayed after ${waitInMilliseconds} milliseconds.`;
                utils_1.logInfo(msg);
            }
            return element;
        });
    }
    /**
     * Search for elements by given automationText. Searches only for exact string. Returns promise with array of elements.
     * @param text
     * @param waitForElement
     */
    findElementsByAutomationText(automationText, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isIOS) {
                return yield this.findElementsByAccessibilityId(`${automationText}`);
            }
            return yield this.findElementsByXPath(this._elementHelper.getXPathByText(automationText, true), waitForElement);
        });
    }
    /**
     * Search for elements by given text. The search is case insensitive for android
     * @param text
     * @param match
     * @param waitForElement
     */
    findElementsByText(text, match = search_options_1.SearchOptions.exact, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldMatch = match === search_options_1.SearchOptions.exact ? true : false;
            return yield this.findElementsByXPath(this._elementHelper.getXPathByText(text, shouldMatch), waitForElement);
        });
    }
    /**
     * Searches for element by element native class name like button, textView etc which will be translated to android.widgets.Button or XCUIElementTypeButton (iOS 10 and higher) or UIElementButton (iOS 9)
     * Notice this is not the same as css class
     * @param className
     * @param waitForElement
     */
    findElementByClassName(className, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            return new ui_element_1.UIElement(yield this._driver.waitForElementByClassName(className, waitForElement), this._driver, this._wd, this._webio, this._args, "waitForElementByClassName", className);
        });
    }
    /**
     * Searches for element by element native class name like button, textView etc which will be translated to android.widgets.Button or XCUIElementTypeButton (iOS 10 and higher) or UIElementButton (iOS 9)
     * Notice this is not the same as css class
     * @param className
     * @param waitForElement
     */
    findElementsByClassName(className, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.convertArrayToUIElements(yield this._driver.waitForElementsByClassName(className, waitForElement), "waitForElementsByClassName", className);
        });
    }
    /**
     * Find element by automationText
     * @param id
     * @param waitForElement
     */
    findElementByAccessibilityId(id, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            return new ui_element_1.UIElement(yield this._driver.waitForElementByAccessibilityId(id, waitForElement), this._driver, this._wd, this._webio, this._args, "waitForElementByAccessibilityId", id);
        });
    }
    /**
     * Find elements by automationText
     * @param id
     * @param waitForElement
     */
    findElementsByAccessibilityId(id, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.convertArrayToUIElements(yield this._driver.waitForElementsByAccessibilityId(id, waitForElement), "waitForElementsByAccessibilityId", id);
        });
    }
    /**
     * Scrolls from point to other point with minimum inertia
     * @param direction
     * @param y
     * @param x
     * @param yOffset
     * @param xOffset
     */
    scroll(direction, y, x, yOffset, xOffset = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            yield utils_1.scroll(this._wd, this._driver, direction, this._webio.isIOS, y, x, yOffset, xOffset, this._args.verbose);
        });
    }
    /**
     *
     * @param direction
     * @param element
     * @param startPoint
     * @param yOffset
     * @param xOffset
     * @param retryCount
     */
    scrollTo(direction, element, startPoint, yOffset, xOffset = 0, retryCount = 7) {
        return __awaiter(this, void 0, void 0, function* () {
            let el = null;
            while (el === null && retryCount > 0) {
                try {
                    el = yield element();
                    if (!(yield el.isDisplayed())) {
                        yield utils_1.scroll(this._wd, this._driver, direction, this._webio.isIOS, startPoint.y, startPoint.x, yOffset, xOffset, this._args.verbose);
                    }
                }
                catch (error) {
                    yield utils_1.scroll(this._wd, this._driver, direction, this._webio.isIOS, startPoint.y, startPoint.x, yOffset, xOffset, this._args.verbose);
                }
                if (el !== null && (yield el.isDisplayed())) {
                    break;
                }
                else {
                    el = null;
                }
                retryCount--;
            }
            return el;
        });
    }
    /**
     * Swipe from point with offset and inertia according to duatio
     * @param y
     * @param x
     * @param yOffset
     * @param inertia
     * @param xOffset
     */
    swipe(y, x, yOffset, inertia = 250, xOffset = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            let direction = 1;
            if (this._webio.isIOS) {
                direction = -1;
            }
            const action = new this._wd.TouchAction(this._driver);
            action
                .press({ x: x, y: y })
                .wait(inertia)
                .moveTo({ x: xOffset, y: direction * yOffset })
                .release();
            yield action.perform();
            yield this._driver.sleep(150);
        });
    }
    /**
    * Click a point by providing coordinates
    * @param x
    * @param y
    */
    clickPoint(xCoordinate, yCoordinate) {
        return __awaiter(this, void 0, void 0, function* () {
            let action = new this._wd.TouchAction(this._driver);
            action
                .tap({ x: xCoordinate, y: yCoordinate });
            yield action.perform();
            yield this._driver.sleep(150);
        });
    }
    source() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._webio.source();
        });
    }
    sessionId() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.driver.getSessionId();
        });
    }
    compareElement(element, imageName, tolerance = 0.01, timeOutSeconds = 3, toleranceType) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.compareRectangle(yield element.getActualRectangle(), imageName, timeOutSeconds, tolerance, toleranceType);
        });
    }
    compareRectangle(rect, imageName, timeOutSeconds = 3, tolerance = 0.01, toleranceType) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.compare(imageName, timeOutSeconds, tolerance, rect, toleranceType);
        });
    }
    compareScreen(imageName, timeOutSeconds = 3, tolerance = 0.01, toleranceType) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.compare(imageName, timeOutSeconds, tolerance, undefined, toleranceType);
        });
    }
    /**
     * @param videoName
     * @param callback when to stop video recording. In order an element is found. Should return true to exit
     */
    recordVideo(videoName, callback) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._storageByDeviceName) {
                this._storageByDeviceName = utils_1.getStorageByDeviceName(this._args);
            }
            return mobile_devices_controller_1.DeviceController.recordVideo(this._args.device, this._storageByDeviceName, videoName, callback);
        });
    }
    /**
     * @param videoName
     */
    startRecordingVideo(videoName) {
        if (!this._logPath) {
            this._logPath = utils_1.getReportPath(this._args);
        }
        videoName = videoName.replace(/\s/gi, "");
        console.log("DEVICE: ", this._args.device);
        this._recordVideoInfo = mobile_devices_controller_1.DeviceController.startRecordingVideo(this._args.device, this._logPath, videoName);
        this._recordVideoInfo['device'] = this._args.device;
        return this._recordVideoInfo['pathToVideo'];
    }
    stopRecordingVideo() {
        this._recordVideoInfo['videoRecoringProcess'].kill("SIGINT");
        utils_1.wait(this.isIOS ? 100 : 10000);
        if (this._args.device.type === mobile_devices_controller_1.DeviceType.EMULATOR || this._args.device.platform === mobile_devices_controller_1.Platform.ANDROID) {
            mobile_devices_controller_1.AndroidController.pullFile(this._recordVideoInfo['device'], this._recordVideoInfo['devicePath'], this._recordVideoInfo['pathToVideo'].endsWith(".mp4") ? this._recordVideoInfo['pathToVideo'] : `${this._recordVideoInfo['pathToVideo']}.mp4`);
            utils_1.wait(20000);
        }
        return Promise.resolve(this._recordVideoInfo['pathToVideo']);
    }
    compare(imageName, timeOutSeconds = 3, tolerance = 0.01, rect, toleranceType) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._logPath) {
                this._logPath = utils_1.getReportPath(this._args);
            }
            imageName = utils_1.addExt(imageName, AppiumDriver.pngFileExt);
            const pathExpectedImage = this.getExpectedImagePath(imageName);
            // First time capture
            if (!fs_1.existsSync(pathExpectedImage)) {
                const pathActualImage = utils_1.resolvePath(this._storageByDeviceName, imageName.replace(".", "_actual."));
                yield this.takeScreenshot(pathActualImage);
                if (rect) {
                    yield this._imageHelper.clipRectangleImage(rect, pathActualImage);
                }
                const pathActualImageToReportsFolder = utils_1.resolvePath(this._logPath, path_1.basename(pathActualImage));
                utils_1.copy(pathActualImage, pathActualImageToReportsFolder, false);
                console.log("Remove the 'actual' suffix to continue using the image as expected one ", pathExpectedImage);
                return false;
            }
            // Compare
            let pathActualImage = yield this.takeScreenshot(utils_1.resolvePath(this._logPath, imageName.replace(".", "_actual.")));
            const pathDiffImage = pathActualImage.replace("actual", "diff");
            yield this.prepareImageToCompare(pathActualImage, rect);
            let result = yield this._imageHelper.compareImages(pathActualImage, pathExpectedImage, pathDiffImage, tolerance, toleranceType);
            // Iterate
            if (!result) {
                const eventStartTime = Date.now().valueOf();
                let counter = 1;
                timeOutSeconds *= 1000;
                while ((Date.now().valueOf() - eventStartTime) <= timeOutSeconds && !result) {
                    const pathActualImageConter = utils_1.resolvePath(this._logPath, imageName.replace(".", "_actual_" + counter + "."));
                    pathActualImage = yield this.takeScreenshot(pathActualImageConter);
                    yield this.prepareImageToCompare(pathActualImage, rect);
                    result = yield this._imageHelper.compareImages(pathActualImage, pathExpectedImage, pathDiffImage, tolerance, toleranceType);
                    counter++;
                }
            }
            else {
                if (fs_1.existsSync(pathDiffImage)) {
                    fs_1.unlinkSync(pathDiffImage);
                }
                if (fs_1.existsSync(pathActualImage)) {
                    fs_1.unlinkSync(pathActualImage);
                }
            }
            this._imageHelper.imageCropRect = undefined;
            return result;
        });
    }
    prepareImageToCompare(filePath, rect) {
        return __awaiter(this, void 0, void 0, function* () {
            if (rect) {
                yield this._imageHelper.clipRectangleImage(rect, filePath);
                const rectToCrop = { x: 0, y: 0, width: undefined, height: undefined };
                this._imageHelper.imageCropRect = rectToCrop;
            }
            else {
                this._imageHelper.imageCropRect = image_helper_1.ImageHelper.cropImageDefault(this._args);
            }
        });
    }
    takeScreenshot(fileName) {
        if (!fileName.endsWith(AppiumDriver.pngFileExt)) {
            fileName = fileName.concat(AppiumDriver.pngFileExt);
        }
        return new Promise((resolve, reject) => {
            this._driver.takeScreenshot().then(function (image, err) {
                if (err) {
                    console.error(err);
                    reject(err);
                }
                fs_1.writeFileSync(fileName, image, 'base64');
                resolve(fileName);
            });
        });
    }
    logScreenshot(fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._logPath) {
                this._logPath = utils_1.getReportPath(this._args);
            }
            if (!fileName.endsWith(AppiumDriver.pngFileExt)) {
                fileName = fileName.concat(AppiumDriver.pngFileExt);
            }
            const imgPath = yield this.takeScreenshot(utils_1.resolvePath(this._logPath, fileName));
            return imgPath;
        });
    }
    getlog(logType) {
        return __awaiter(this, void 0, void 0, function* () {
            const logs = yield this._driver.log(logType);
            return logs;
        });
    }
    logPageSource(fileName) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._logPath) {
                this._logPath = utils_1.getReportPath(this._args);
            }
            if (!fileName.endsWith(".xml")) {
                fileName = fileName.concat(".xml");
            }
            const path = utils_1.resolvePath(this._logPath, fileName);
            const xml = yield this.source();
            fs_1.writeFileSync(path, xml.value, 'utf8');
        });
    }
    logDeviceLog(fileName, logType, filter = undefined) {
        return __awaiter(this, void 0, void 0, function* () {
            let logs;
            try {
                logs = yield this.getlog(logType);
            }
            catch (error) {
                utils_1.logError(`Failed to get log type: ${logType}`);
            }
            let deviceLog = "";
            logs.forEach(log => {
                const curruntLog = `\n${JSON.stringify(log)}`;
                if (filter) {
                    if (curruntLog.includes(filter)) {
                        deviceLog += `\n${JSON.stringify(log)}`;
                    }
                }
                else {
                    deviceLog += `\n${JSON.stringify(log)}`;
                }
            });
            if (logs.length > 0 && deviceLog) {
                const ext = path_1.extname(fileName);
                fileName = fileName.replace(ext, "");
                fileName = fileName.concat('_').concat(logType);
                fileName = fileName.concat(".log");
                if (!this._logPath) {
                    this._logPath = utils_1.getReportPath(this._args);
                }
                const path = utils_1.resolvePath(this._logPath, fileName);
                fs_1.writeFileSync(path, deviceLog, 'utf8');
            }
            else {
                console.log(`Log type: ${logType} is empty!`);
            }
        });
    }
    /**
     * This method will snapshot the screen of device, get page source and log from device
     * @param logName
     */
    logTestArtifacts(logName) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.logScreenshot(logName);
            yield this.logPageSource(logName);
            if (this.isAndroid) {
                yield this.logDeviceLog(logName, log_types_1.LogType.logcat);
            }
            else {
                yield this.logDeviceLog(logName, log_types_1.LogType.crashlog);
                yield this.logDeviceLog(logName, log_types_1.LogType.syslog);
            }
        });
    }
    /**
     * Send the currently active app to the background
     * @param time in minutes
     */
    backgroundApp(minutes) {
        return __awaiter(this, void 0, void 0, function* () {
            utils_1.logInfo("Sending the currently active app to the background ...");
            yield this._driver.backgroundApp(minutes);
        });
    }
    /**
     * Hides device keyboard
     */
    hideDeviceKeyboard() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._driver.hideDeviceKeyboard();
        });
    }
    isKeyboardShown() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this._driver.isKeyboardShown();
        });
    }
    resetApp() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._driver.resetApp();
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._driver.init(this._args.appiumCaps);
            this._webio.requestHandler.sessionID = this._driver.sessionID;
            this._isAlive = true;
        });
    }
    quit() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._recordVideoInfo && this._recordVideoInfo['videoRecordingProcess']) {
                this._recordVideoInfo['videoRecordingProcess'].kill("SIGINT");
            }
            try {
                if (!this._args.attachToDebug) {
                    console.log("Killing driver...");
                    yield this._driver.quit();
                    this._isAlive = false;
                    console.log("Driver is dead!");
                }
                else {
                    //await this._webio.detach();
                }
            }
            catch (error) {
                if (this._args.verbose) {
                    console.dir(error);
                }
            }
        });
    }
    static applyAdditionalSettings(args) {
        return __awaiter(this, void 0, void 0, function* () {
            if (args.isSauceLab)
                return;
            args.appiumCaps['udid'] = args.appiumCaps['udid'] || args.device.token;
            if (args.device.type === mobile_devices_controller_1.DeviceType.EMULATOR && args.device.token) {
                args.appiumCaps['udid'] = args.device.token.startsWith("emulator") ? args.device.token : `emulator-${args.device.token}`;
            }
            if (!args.appiumCaps['udid']) {
                delete args.appiumCaps['udid'];
            }
            if (args.appiumCaps.platformName.toLowerCase() === mobile_devices_controller_1.Platform.IOS) {
                args.appiumCaps["useNewWDA"] = args.appiumCaps.useNewWDA;
                args.appiumCaps["wdaStartupRetries"] = 5;
                args.appiumCaps["shouldUseSingletonTestManager"] = args.appiumCaps.shouldUseSingletonTestManager;
                // It looks we need it for XCTest (iOS 10+ automation)
                if (args.appiumCaps.platformVersion >= 10 && args.wdaLocalPort) {
                    console.log(`args.appiumCaps['wdaLocalPort']: ${args.wdaLocalPort}`);
                    args.appiumCaps["wdaLocalPort"] = args.wdaLocalPort;
                }
            }
            else {
                if (process.env["SYSTEM_PORT"]) {
                    args.appiumCaps['systemPort'] = process.env["SYSTEM_PORT"];
                }
            }
        });
    }
    convertArrayToUIElements(array, searchM, args) {
        return __awaiter(this, void 0, void 0, function* () {
            const arrayOfUIElements = new Array();
            if (!array || array === null) {
                return arrayOfUIElements;
            }
            for (let index = 0; index < array.length; index++) {
                const element = array[index];
                arrayOfUIElements.push(new ui_element_1.UIElement(yield element, this._driver, this._wd, this._webio, this._args, searchM, args, index));
            }
            return arrayOfUIElements;
        });
    }
    static configureLogging(driver, verbose) {
        driver.on("status", function (info) {
            utils_1.log(info, verbose);
        });
        driver.on("quit", function (info) {
            console.log("QUIT: ", info);
        });
        driver.on("command", function (meth, path, data) {
            utils_1.log(" > " + meth + " " + path + " " + (data || ""), verbose);
        });
        driver.on("http", function (meth, path, data) {
            utils_1.log(" > " + meth + " " + path + " " + (data || ""), verbose);
        });
    }
    ;
    getExpectedImagePath(imageName) {
        if (!this._storageByDeviceName) {
            this._storageByDeviceName = utils_1.getStorageByDeviceName(this._args);
        }
        let pathExpectedImage = utils_1.resolvePath(this._storageByDeviceName, imageName);
        if (!fs_1.existsSync(pathExpectedImage)) {
            if (!this._storageByPlatform) {
                this._storageByPlatform = utils_1.getStorageByPlatform(this._args);
            }
            pathExpectedImage = utils_1.resolvePath(this._storageByPlatform, imageName);
        }
        if (!fs_1.existsSync(pathExpectedImage)) {
            pathExpectedImage = utils_1.resolvePath(this._storageByDeviceName, imageName);
        }
        return pathExpectedImage;
    }
    /**
    * Wait specific amount of time before continue execution
    * @param milliseconds
    */
    sleep(milliseconds) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._driver.sleep(milliseconds);
        });
    }
    /**
  * Wait specific amount of time before continue execution
  * @param milliseconds
  */
    wait(milliseconds) {
        utils_1.wait(milliseconds);
    }
    /**
    * Search for element by given xPath but does not throw error if can not find it. Instead returns 'undefined'.
    * @param xPath
    * @param waitForElement
    */
    findElementByXPathIfExists(xPath, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            const element = yield this._driver.elementByXPathIfExists(xPath, waitForElement);
            if (element) {
                const searchMethod = "elementByXPathIfExists";
                return yield new ui_element_1.UIElement(element, this._driver, this._wd, this._webio, this._args, searchMethod, xPath);
            }
            else {
                return undefined;
            }
        });
    }
    /**
    * Search for element by given text but does not throw error if can not find it. Instead returns 'undefined'.
    * @param text
    * @param match
    * @param waitForElement
    */
    findElementByTextIfExists(text, match = search_options_1.SearchOptions.exact, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            const shouldMatch = match === search_options_1.SearchOptions.exact ? true : false;
            return yield this.findElementByXPathIfExists(this._elementHelper.getXPathByText(text, shouldMatch), waitForElement);
        });
    }
    /**
    * Search for element by automationText but does not throw error if can not find it. Instead returns 'undefined'.
    * @param id
    * @param waitForElement
    */
    findElementByAccessibilityIdIfExists(id, waitForElement = this.defaultWaitTime) {
        return __awaiter(this, void 0, void 0, function* () {
            let element = undefined;
            try {
                element = yield this._driver.elementByAccessibilityIdIfExists(id, waitForElement);
            }
            catch (error) { }
            if (element) {
                const searchMethod = "elementByAccessibilityIdIfExists";
                return yield new ui_element_1.UIElement(element, this._driver, this._wd, this._webio, this._args, searchMethod, id);
            }
            return element;
        });
    }
    setDontKeepActivities(value) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._args.isAndroid) {
                const output = yield device_manager_1.DeviceManager.setDontKeepActivities(this._args, this._driver, value);
            }
            else {
                // Do nothing for iOS ...
            }
        });
    }
    /**
     *  Experimental feature that is still tricky to use!!!
     *  Find location on the screen by provided image.
     * @param image The name of the image without the extension.
     * @param imageThreshold The degree of match for current search, on the scale between 0 and 1. Default 0.4
     */
    findElementByImage(image, imageThreshold = 0.4) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._driver.updateSettings({ imageMatchThreshold: imageThreshold });
            const imageName = utils_1.addExt(image, AppiumDriver.pngFileExt);
            const pathExpectedImage = this.getExpectedImagePath(imageName);
            if (!fs_1.existsSync(pathExpectedImage)) {
                throw new Error("The provided image does not exist!!!");
            }
            const imageAsBase64 = utils_1.encodeImageToBase64(pathExpectedImage);
            let searchResult;
            try {
                searchResult = yield this._driver.elementByImage(imageAsBase64);
            }
            catch (error) {
                throw new Error(error);
            }
            finally {
                // reset setting to default value
                yield this._driver.updateSettings({ imageMatchThreshold: 0.4 });
            }
            return new ui_element_1.UIElement(searchResult, this._driver, this._wd, this._webio, this._args, "elementByImage", imageAsBase64);
        });
    }
}
AppiumDriver.pngFileExt = '.png';
AppiumDriver.partialUrl = "/wd/hub/session/";
exports.AppiumDriver = AppiumDriver;
//# sourceMappingURL=appium-driver.js.map